"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFormattedSelectMeta = void 0;
const grammarCases = [
    'ablative',
    'absolutive',
    'accusative',
    'adessive',
    'adverbial',
    'agentive',
    'allative',
    'antessive',
    'benefactive',
    'causal',
    'comitative',
    'dative',
    'delative',
    'distributive',
    'elative',
    'ergative',
    'essive',
    'formal',
    'genitive',
    'illative',
    'inessive',
    'initiative',
    'instructive',
    'instrumental',
    'intransitive',
    'lative',
    'limitative',
    'locative',
    'nominative',
    'objective',
    'oblique',
    'ornative',
    'partitive',
    'privative',
    'possessive',
    'prepositional',
    'prolative',
    'sociative',
    'sublative',
    'superessive',
    'superlative',
    'temporal',
    'terminative',
    'translative',
    'vocative'
];
const genders = ['common', 'feminine', 'masculine', 'neuter'];
const plurals = ['zero', 'one', 'two', 'few', 'many', 'other'];
const isNumeric = (str) => Number.isFinite(Number(str));
function getFormattedSelectMeta(selectMsg, keys) {
    const det = detectGrammarSelectors(selectMsg);
    let gcase;
    if (det.case !== -1) {
        const key = keys[det.case];
        if (key.type !== '*')
            gcase = key.value;
    }
    let gender;
    if (det.gender !== -1) {
        const key = keys[det.gender];
        if (key.type !== '*')
            gender = key.value;
    }
    let plural;
    if (det.plural !== -1) {
        const key = keys[det.plural];
        if (key.type !== '*')
            plural = key.value;
    }
    if (gcase || gender || plural) {
        const meta = {};
        if (gcase)
            meta.case = gcase;
        if (gender)
            meta.gender = gender;
        if (plural)
            meta.plural = plural;
        return meta;
    }
    return undefined;
}
exports.getFormattedSelectMeta = getFormattedSelectMeta;
/**
 * Duck-type selectors based on the keys used to match them.
 *
 * Detects grammatical cases, grammatical genders, and plural categories.
 *
 * @returns Indices of first matching selectors, or -1 for no match.
 */
function detectGrammarSelectors({ selectors, variants }) {
    const gc = new Array(selectors.length).fill(null);
    for (const { keys } of variants) {
        for (let i = 0; i < gc.length; ++i) {
            const c = gc[i];
            if (c === 4 /* GC.Other */)
                continue;
            const key = keys[i];
            if (!key || key.type === '*')
                continue;
            if (isNumeric(key.value) || plurals.includes(key.value)) {
                if (c !== 3 /* GC.Plural */)
                    gc[i] = c ? 4 /* GC.Other */ : 3 /* GC.Plural */;
            }
            else if (grammarCases.includes(key.value)) {
                if (c !== 1 /* GC.Case */)
                    gc[i] = c ? 4 /* GC.Other */ : 1 /* GC.Case */;
            }
            else if (genders.includes(key.value)) {
                if (c !== 2 /* GC.Gender */)
                    gc[i] = c ? 4 /* GC.Other */ : 2 /* GC.Gender */;
            }
            else {
                gc[i] = 4 /* GC.Other */;
            }
        }
    }
    return {
        case: gc.indexOf(1 /* GC.Case */),
        gender: gc.indexOf(2 /* GC.Gender */),
        plural: gc.indexOf(3 /* GC.Plural */)
    };
}
