"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageNumber = void 0;
const locale_context_1 = require("./locale-context");
const message_value_1 = require("./message-value");
/**
 * A child class of {@link MessageValue} for numerical values.
 *
 * @beta
 */
class MessageNumber extends message_value_1.MessageValue {
    constructor(locale, number, { meta, options, source } = {}) {
        const fmt = { meta, source };
        if (number instanceof MessageNumber) {
            const lc = (0, locale_context_1.extendLocaleContext)(number.localeContext, locale);
            super(MessageNumber.type, lc, number.value, fmt);
            if (options || number.options)
                this.options = number.options
                    ? Object.assign(Object.assign({}, number.options), options) : Object.assign({}, options);
        }
        else if (typeof number === 'number' || typeof number === 'bigint') {
            super(MessageNumber.type, locale, number, fmt);
            if (options)
                this.options = Object.assign({}, options);
        }
        else
            throw TypeError(`Invalid ${typeof number} as number argument`);
    }
    getIntl(Class) {
        const lc = this.localeContext;
        const lm = lc === null || lc === void 0 ? void 0 : lc.localeMatcher;
        const opt = lm ? Object.assign({ localeMatcher: lm }, this.options) : this.options;
        return new Class(lc === null || lc === void 0 ? void 0 : lc.locales, opt);
    }
    getPluralCategory() {
        if (!this.localeContext)
            return 'other';
        const pr = this.getIntl(Intl.PluralRules);
        // Intl.PluralRules really does need a number
        const num = Number(this.value);
        return pr.select(num);
    }
    /**
     * In addition to matching exact values,
     * numerical values will also match keys with the same plural rule category,
     * i.e. one of `zero`, `one`, `two`, `few`, `many`, and `other`.
     *
     * Different languages use different subset of plural rule categories.
     * For example, cardinal English plurals only use `one` and `other`,
     * so a key `zero` will never be matched for that locale.
     */
    matchSelectKey(key) {
        return ((/^[0-9]+$/.test(key) && key === String(this.value)) ||
            key === this.getPluralCategory());
    }
    toParts() {
        const nf = this.getIntl(Intl.NumberFormat);
        return nf.formatToParts(this.value);
    }
    toString(onError) {
        try {
            const nf = this.getIntl(Intl.NumberFormat);
            return nf.format(this.value);
        }
        catch (error) {
            if (onError)
                onError(error, this);
            if (this.value === undefined) {
                const source = this.source || message_value_1.FALLBACK_SOURCE;
                return `{${source}}`;
            }
            return String(this.value);
        }
    }
}
exports.MessageNumber = MessageNumber;
MessageNumber.type = 'number';
