"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _ResolvedMessage_str;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResolvedMessage = void 0;
const detect_grammar_1 = require("../extra/detect-grammar");
const message_markup_1 = require("./message-markup");
const message_value_1 = require("./message-value");
function getPattern(context, message) {
    switch (message.type) {
        case 'message':
            return { pattern: message.pattern.body };
        case 'select': {
            const resSelectors = message.selectors.map(sel => context.resolve(sel));
            cases: for (const { keys, value } of message.variants) {
                for (let i = 0; i < keys.length; ++i) {
                    const key = keys[i];
                    const rs = resSelectors[i];
                    if (key.type !== '*' && rs && !rs.matchSelectKey(key.value)) {
                        continue cases;
                    }
                }
                const meta = (0, detect_grammar_1.getFormattedSelectMeta)(message, keys);
                return { pattern: value.body, meta };
            }
            return { pattern: [], meta: { selectResult: 'no-match' } };
        }
        default:
            return { pattern: [{ type: 'junk', source: message.source }] };
    }
}
/**
 * The result of resolving a {@link MessageFormat} message.
 *
 * @beta
 */
class ResolvedMessage extends message_value_1.MessageValue {
    constructor(context, message, source) {
        const { meta, pattern } = getPattern(context, message);
        const resMsg = pattern.map(elem => context.resolve(elem));
        (0, message_markup_1.fillMessageMarkups)(resMsg);
        super(ResolvedMessage.type, context, resMsg, { meta, source });
        // Cache for string value
        _ResolvedMessage_str.set(this, void 0);
    }
    matchSelectKey(key) {
        return this.toString() === key;
    }
    toString(onError, noCache = false) {
        if (noCache || typeof __classPrivateFieldGet(this, _ResolvedMessage_str, "f") !== 'string') {
            __classPrivateFieldSet(this, _ResolvedMessage_str, '', "f");
            for (const mv of this.value)
                __classPrivateFieldSet(this, _ResolvedMessage_str, __classPrivateFieldGet(this, _ResolvedMessage_str, "f") + mv.toString(onError), "f");
        }
        return __classPrivateFieldGet(this, _ResolvedMessage_str, "f");
    }
}
exports.ResolvedMessage = ResolvedMessage;
_ResolvedMessage_str = new WeakMap();
ResolvedMessage.type = 'message';
