"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseVariable = exports.parseLiteral = exports.parseText = void 0;
const names_js_1 = require("./names.js");
// Text ::= (TextChar | TextEscape)+
// TextChar ::= AnyChar - ('{' | '}' | Esc)
// AnyChar ::= [#x0-#x10FFFF]
// Esc ::= '\'
// TextEscape ::= Esc Esc | Esc '{' | Esc '}'
function parseText(src, start, errors) {
    let value = '';
    let pos = start;
    let i = start;
    loop: for (; i < src.length; ++i) {
        switch (src[i]) {
            case '\\': {
                const esc = src[i + 1];
                if (esc !== '\\' && esc !== '{' && esc !== '}') {
                    errors.push({ type: 'bad-escape', start: i, end: i + 2 });
                }
                else {
                    value += src.slice(pos, i);
                    i += 1;
                    pos = i;
                }
                break;
            }
            case '{':
            case '}':
                break loop;
        }
    }
    value += src.slice(pos, i);
    return { type: 'text', start, end: i, value };
}
exports.parseText = parseText;
// Literal ::= '(' (LiteralChar | LiteralEscape)* ')' /* ws: explicit */
// Esc ::= '\'
// LiteralChar ::= AnyChar - ('(' | ')' | Esc)
// LiteralEscape ::= Esc Esc | Esc '(' | Esc ')'
function parseLiteral(src, start, errors) {
    let value = '';
    let pos = start + 1;
    for (let i = pos; i < src.length; ++i) {
        switch (src[i]) {
            case '\\': {
                const esc = src[i + 1];
                if (esc !== '\\' && esc !== '(' && esc !== ')') {
                    errors.push({ type: 'bad-escape', start: i, end: i + 2 });
                }
                else {
                    value += src.substring(pos, i);
                    i += 1;
                    pos = i;
                }
                break;
            }
            case '(':
                errors.push({ type: 'missing-char', char: '\\', start: i });
                break;
            case ')':
                value += src.substring(pos, i);
                return { type: 'literal', start, end: i + 1, value };
        }
    }
    value += src.substring(pos);
    errors.push({ type: 'missing-char', char: ')', start: src.length });
    return { type: 'literal', start, end: src.length, value };
}
exports.parseLiteral = parseLiteral;
// Variable ::= '$' Name /* ws: explicit */
function parseVariable(src, start, errors) {
    const pos = start + 1;
    const name = (0, names_js_1.parseNameValue)(src, pos);
    const end = pos + name.length;
    if (!name)
        errors.push({ type: 'empty-token', start: pos });
    return { type: 'variable', start, end, name };
}
exports.parseVariable = parseVariable;
