import type { Context } from '../format-context';
import { MessageValue } from '../message-value';
import type { Literal, VariableRef } from './index';
/**
 * To resolve an Expression, an externally defined function is called.
 *
 * The `name` identifies a function that takes in the arguments `args`, the
 * current locale, as well as any `options`, and returns some corresponding
 * output. Likely functions available by default would include `'plural'` for
 * determining the plural category of a numeric value, as well as `'number'`
 * and `'date'` for formatting values.
 *
 * @beta
 */
export interface Expression {
    type: 'expression';
    name: string;
    operand?: Literal | VariableRef;
    options?: Option[];
}
/**
 * {@link Expression} and {@link MarkupStart} options are expressed as
 * `key`/`value` pairs to allow their order to be maintained.
 *
 * @beta
 */
export interface Option {
    name: string;
    value: Literal | VariableRef;
}
/**
 * Type guard for {@link Expression} pattern elements
 *
 * @beta
 */
export declare const isExpression: (part: any) => part is Expression;
export declare function resolveExpression(ctx: Context, { operand, name, options }: Expression): MessageValue<unknown>;
