import type { Context } from '../format-context';
import type { MessageValue } from '../message-value';
import { Expression } from './expression';
import { Junk } from './junk';
import { Literal, Text } from './literal';
import { MarkupEnd, MarkupStart } from './markup';
import { VariableRef } from './variable-ref';
export { isExpression, Expression, Option } from './expression';
export { isJunk, Junk } from './junk';
export { isLiteral, Literal, Text } from './literal';
export { isMarkupEnd, isMarkupStart, MarkupEnd, MarkupStart } from './markup';
export { isVariableRef, VariableRef } from './variable-ref';
/**
 * Wrapper for non-literal content.
 *
 * @beta
 */
export interface Placeholder {
    type: 'placeholder';
    body: Literal | VariableRef | Expression | MarkupStart | MarkupEnd | Junk;
}
/**
 * The contents of a message are a sequence of pattern elements, which may be
 * immediately defined literal values, a reference to a value that depends on
 * another message, the value of some runtime variable, or some function
 * defined elsewhere.
 *
 * Depending on the syntax, pattern elements may be wrapped within a Placeholder.
 *
 * @beta
 */
export declare type PatternElement = Expression | Junk | Literal | MarkupEnd | MarkupStart | Placeholder | Text | VariableRef;
/** @internal */
export declare function resolvePatternElement(ctx: Context, elem: PatternElement): MessageValue;
