"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveMarkupEnd = exports.resolveMarkupStart = exports.isMarkupEnd = exports.isMarkupStart = void 0;
const message_value_1 = require("../message-value");
/**
 * Type guard for {@link MarkupStart} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isMarkupStart = (part) => !!part && typeof part === 'object' && part.type === 'markup-start';
exports.isMarkupStart = isMarkupStart;
/**
 * Type guard for {@link MarkupEnd} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isMarkupEnd = (part) => !!part && typeof part === 'object' && part.type === 'markup-end';
exports.isMarkupEnd = isMarkupEnd;
function resolveOptions(ctx, options) {
    const opt = {};
    if (options) {
        for (const { name, value } of options) {
            opt[name] = ctx.resolve(value).value;
        }
    }
    return opt;
}
function resolveMarkupStart(ctx, { name, options }) {
    const source = `<${name}>`;
    try {
        return new message_value_1.MessageMarkup(ctx, name, {
            options: resolveOptions(ctx, options),
            source,
            tag: 'start'
        });
    }
    catch (error) {
        const fb = new message_value_1.MessageFallback(ctx, { source });
        ctx.onError(error, fb);
        return fb;
    }
}
exports.resolveMarkupStart = resolveMarkupStart;
function resolveMarkupEnd(ctx, { name }) {
    return new message_value_1.MessageMarkup(ctx, name, { source: `</${name}>`, tag: 'end' });
}
exports.resolveMarkupEnd = resolveMarkupEnd;
