"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.number = exports.datetime = void 0;
const message_value_1 = require("../message-value");
exports.datetime = {
    call: function datetime(locales, options, arg) {
        let date;
        if (!arg)
            date = new Date();
        else if (arg instanceof message_value_1.MessageDateTime)
            date = arg;
        else if (typeof arg.value === 'number')
            date = new Date(arg.value);
        else
            date = new Date(String(arg.value));
        return new message_value_1.MessageDateTime(locales, date, { options });
    },
    options: {
        localeMatcher: ['best fit', 'lookup'],
        weekday: ['long', 'short', 'narrow'],
        era: ['long', 'short', 'narrow'],
        year: ['numeric', '2-digit'],
        month: ['numeric', '2-digit', 'long', 'short', 'narrow'],
        day: ['numeric', '2-digit'],
        hour: ['numeric', '2-digit'],
        minute: ['numeric', '2-digit'],
        second: ['numeric', '2-digit'],
        timeZoneName: ['long', 'short'],
        formatMatcher: ['best fit', 'basic'],
        hour12: 'boolean',
        timeZone: 'string',
        // ES 2020
        dateStyle: ['full', 'long', 'medium', 'short'],
        timeStyle: ['full', 'long', 'medium', 'short'],
        calendar: 'string',
        dayPeriod: ['narrow', 'short', 'long'],
        numberingSystem: 'string',
        hourCycle: ['h11', 'h12', 'h23', 'h24'],
        fractionalSecondDigits: 'number' // 0 | 1 | 2 | 3
    }
};
exports.number = {
    call: function number(locales, options, arg) {
        const num = arg instanceof message_value_1.MessageNumber ? arg : Number(arg === null || arg === void 0 ? void 0 : arg.value);
        return new message_value_1.MessageNumber(locales, num, { options });
    },
    options: {
        localeMatcher: ['best fit', 'lookup'],
        style: 'string',
        currency: 'string',
        currencyDisplay: 'string',
        currencySign: 'string',
        useGrouping: 'boolean',
        minimumIntegerDigits: 'number',
        minimumFractionDigits: 'number',
        maximumFractionDigits: 'number',
        minimumSignificantDigits: 'number',
        maximumSignificantDigits: 'number',
        // ES 2020
        compactDisplay: 'string',
        notation: 'string',
        signDisplay: 'string',
        unit: 'string',
        unitDisplay: 'string',
        // Intl.PluralRules
        type: ['cardinal', 'ordinal']
    }
};
